# 
# searching for optimal SVM parameters 
#
# Author: Julien Gagneur
###############################################################################

## load functions
source("R/svm_funcs.R")
source("R/bestAUC.R")

## regular log sequence
logseq = function(a,b,n=8) exp(seq(log(a), log(b), length.out=n))

#####################################################
# settings
#####################################################
folder = "data"

# maximal proportion of support vectors
max.prop.SV = 0.9

#####################################################
# load data
#####################################################
tab8 = read.delim( file.path(folder, "supplementary_table_8_training_set.txt"), comment.char="#" )
elem2insitu = tab8[, 1:11]
x = as.matrix( tab8[, 12:ncol(tab8) ] )

tissues = c("Meso", "VM" , "SM", "Meso_SM", "VM_SM")

#####################################################
# primary grid fit
#####################################################
plot.folder = file.path(folder, "plot_primary" )
if(!file.exists(plot.folder)) dir.create(plot.folder)

# primary grid
cost = logseq(1e-2, 1e+5)
gamma = logseq(1e-6, 1e2)
params = expand.grid(cost=cost, gamma=gamma)

print("Primary grid:")
for(ti in tissues){
    cat("treating", ti, "\n")
    auc = bestAUC(params, x, elem2insitu, ti, kernel="radial", max.prop.SV=max.prop.SV, verbose=TRUE, plot.folder=plot.folder )
}

#####################################################
# from the plot of the primary grid fit, a secondary (zoomed)
# grid was manually defined.
#
# The boundaries of the secondary grid have been stored in the file secgrid.rda
#
# This is now read and the function bestAUC called again 
#####################################################
plot.folder = file.path(folder, "plot_secondary" )
if(!file.exists(plot.folder)) dir.create(plot.folder)

# reads secondary grid boundaries
load( file.path ( folder, "secgrid.rda") )

print("Secondary grid:")
auc = list()
for(i in 1:nrow(secgrid)){
    ti = secgrid$tissue[i]
    
    cat("treating", ti, "\n")
    # make 2nd grid
    cost = logseq(secgrid$cmin[i], secgrid$cmax[i])
    gamma = logseq(secgrid$gmin[i], secgrid$gmax[i])
    params = expand.grid(cost=cost, gamma=gamma)
    
    auc[[ti]] = bestAUC(params, x, elem2insitu, ti, kernel="radial", max.prop.SV=max.prop.SV, nbest=10, verbose=TRUE, plot.folder=plot.folder )
}

#####################################################
# AUC values for all parameters of the secondary grid are saved in the rdata file auc.rda
# Note that the final SVM has been chosen not only according the AUC criteria but, amont the top 10 AUC,
# for showing high sensitivities in the 80-100 % specificity range (see plots in plot.folder for ROC plots of the top 10)
#
# The finally selected SVM parameters have been stored in the file "svmParams.rda"  
#####################################################
save( auc, file=file.path(folder, "auc.rda") )


